package it.unina.instrumented.korel.triangle;

public class KorelFunction {

	public static final String EQUAL_OP = "eq";
	public static final String NOT_EQUAL_OP = "!eq";
	public static final String LESS_THEN_OP = "lt";
	public static final String LESS_EQUAL_OP = "le";
	public static final String GREATER_THEN_OP = "gt";
	public static final String GREATER_EQUAL_OP = "ge";

	/**
	 * calcola la funzione di Korel dati due valori in base all'operatore
	 * booleano con cui sono confrontati
	 * 
	 * @param val1
	 *            primo valore
	 * @param val2
	 *            secondo valore
	 * @param op
	 *            operatore booleano
	 * @return corrispondente valore della funzione di korel
	 */
	public static final Double korelFunction(Double val1, Double val2, String op) {
		Double korelValue = null;

		if (op.equals(EQUAL_OP)) {
			korelValue = Math.abs(val1 - val2);
		} else if (op.equals(NOT_EQUAL_OP)) {
			if (val1.equals(val2))
				korelValue = Double.MAX_VALUE;
			else
				korelValue = 0d;
		} else if (op.equals(LESS_THEN_OP)) {
			if (val1.equals(val2))
				korelValue = 1 / Double.MAX_VALUE;
			else if (val1 > val2)
				korelValue = val1 - val2;
			else
				korelValue = 0d;
		} else if (op.equals(LESS_EQUAL_OP)) {
			if (val1 > val2)
				korelValue = val1 - val2;
			else
				korelValue = 0d;
		} else if (op.equals(GREATER_THEN_OP)) {
			if (val1.equals(val2))
				korelValue = 1 / Double.MAX_VALUE;
			else if (val2 > val1)
				korelValue = val2 - val1;
			else
				korelValue = 0d;
		} else if (op.equals(GREATER_EQUAL_OP)) {
			if (val2 > val1)
				korelValue = val2 - val1;
			else
				korelValue = 0d;
		} else {
			// TODO: gestione errore
		}
		
//		System.out.println(val1 +" "+ op + " " + val2 + "=" + korelValue);
		return korelValue;
	}

	public static final Double korelFunction(Integer val1, Integer val2,
			String op) {
		return korelFunction(new Double(val1), new Double(val2), op);
	}
	
	public static final Double korelFunction(Double val1, Integer val2,
			String op) {
		return korelFunction(new Double(val1), new Double(val2), op);
	}
	
	public static final Double korelFunction(Integer val1, Double val2,
			String op) {
		return korelFunction(new Double(val1), new Double(val2), op);
	}
	
	public static final Double korelFunction(Boolean val) {
		if (val) return 0d;
		return Double.MAX_VALUE;
	}
}
