package it.unina.instrumented.korel.triangle;

/**
 * Classe che fornisce la "funzione triangolo" da testare
 * 
 * @author francesco
 * 
 */
public class Triangle {

	private Integer numTestRequirements = 17;
	private Double[] korelValues = new Double[2 * numTestRequirements];
	private Boolean[] isReached = new Boolean[numTestRequirements];

	public Double[] getKorelValues() {
		return korelValues;
	}

	public Boolean[] getIsReached() {
		return isReached;
	}

	public static final int SCALENE = 1;
	public static final int ISOSCELES = 2;
	public static final int EQUILATERAL = 3;
	public static final int NOTATRIANGLE = 4;

	/**
	 * Dati i 3 lati del triangolo verifica che sia effettivamente un triangolo
	 * e in caso affermativo ne restituisce il tipo
	 * 
	 * @param i
	 *            lunghezza di un lato del triangolo
	 * @param j
	 *            lunghezza di un lato del triangolo
	 * @param k
	 *            lunghezza di un lato del triangolo
	 * @return un intero che identifica il tipo di triangolo - 1 se il triangolo
	 *         è scaleno - 2 se il triangolo è isoscele - 3 se il triangolo è
	 *         equilatero - 4 se non è un triangolo
	 */
	public int triangle(int i, int j, int k) {
		korelValues[0] = KorelFunction.korelFunction(i, 0,
				KorelFunction.LESS_EQUAL_OP);
		korelValues[1] = KorelFunction.korelFunction(i, 0,
				KorelFunction.GREATER_THEN_OP);
		korelValues[2] = KorelFunction.korelFunction(j, 0,
				KorelFunction.LESS_EQUAL_OP);
		korelValues[3] = KorelFunction.korelFunction(j, 0,
				KorelFunction.GREATER_THEN_OP);
		korelValues[4] = KorelFunction.korelFunction(k, 0,
				KorelFunction.LESS_EQUAL_OP);
		korelValues[5] = KorelFunction.korelFunction(k, 0,
				KorelFunction.GREATER_THEN_OP);
		
		if (
				(isReached[0]=(i<=0) && (i <= 0))
				|| 
				(isReached[1]=(j<=0) && (j <= 0))
				|| 
				(isReached[2]=(k<=0) && (k <= 0))
			)
			return 4;

		int tri = 0;

		korelValues[6] = KorelFunction.korelFunction(i, j,
				KorelFunction.EQUAL_OP);
		korelValues[7] = KorelFunction.korelFunction(i, j,
				KorelFunction.NOT_EQUAL_OP);
		if (
				(isReached[3]=(i==j) && (i == j))
			)
			tri += 1;

		korelValues[8] = KorelFunction.korelFunction(i, k,
				KorelFunction.EQUAL_OP);
		korelValues[9] = KorelFunction.korelFunction(i, k,
				KorelFunction.NOT_EQUAL_OP);
		if (
				(isReached[4]=(i==k) && (i == k))
			)
			tri += 2;

		korelValues[10] = KorelFunction.korelFunction(j, k,
				KorelFunction.EQUAL_OP);
		korelValues[11] = KorelFunction.korelFunction(j, k,
				KorelFunction.NOT_EQUAL_OP);
		if (
				(isReached[5]=(j==k) && (j == k))
			)
			tri += 3;

		korelValues[12] = KorelFunction.korelFunction(tri, 0,
				KorelFunction.EQUAL_OP);
		korelValues[13] = KorelFunction.korelFunction(tri, 0,
				KorelFunction.NOT_EQUAL_OP);
		if (
				(isReached[6]=(tri==0) && (tri == 0))
			){
			// In ogni triangolo la somma di 2 lati deve essere maggiore del
			// terzo
			korelValues[14] = KorelFunction.korelFunction(i + j, k,
					KorelFunction.LESS_EQUAL_OP);
			korelValues[15] = KorelFunction.korelFunction(i + j, k,
					KorelFunction.GREATER_THEN_OP);
			korelValues[16] = KorelFunction.korelFunction(j + k, i,
					KorelFunction.LESS_EQUAL_OP);
			korelValues[17] = KorelFunction.korelFunction(j + k, i,
					KorelFunction.GREATER_THEN_OP);
			korelValues[18] = KorelFunction.korelFunction(i + k, j,
					KorelFunction.LESS_EQUAL_OP);
			korelValues[19] = KorelFunction.korelFunction(i + k, j,
					KorelFunction.GREATER_THEN_OP);
			if (
					(isReached[7]=(i + j <= k) && (i + j <= k))
					|| 
					(isReached[8]=(j + k <= i) && (j + k <= i)) 
					|| 
					(isReached[9]=(i + k <= j) && (i + k <= j))
				)
				tri = 4;
			else
				tri = 1;
			return tri;
		}
		korelValues[20] = KorelFunction.korelFunction(tri, 3,
				KorelFunction.GREATER_THEN_OP);
		korelValues[21] = KorelFunction.korelFunction(tri, 3,
				KorelFunction.LESS_EQUAL_OP);
		if (
				(isReached[10]=(tri > 3) && (tri > 3))
			)
			tri = 3;
		else {
			korelValues[22] = KorelFunction.korelFunction(tri, 1,
					KorelFunction.EQUAL_OP);
			korelValues[23] = KorelFunction.korelFunction(tri, 1,
					KorelFunction.NOT_EQUAL_OP);
			korelValues[24] = KorelFunction.korelFunction(i+j, k,
					KorelFunction.GREATER_THEN_OP);
			korelValues[25] = KorelFunction.korelFunction(i+j, k,
					KorelFunction.LESS_EQUAL_OP);
			if (
					(isReached[11]=(tri == 1) && (tri == 1))
					&& 
					(isReached[12]=(i + j > k) && (i + j > k))
				)
				tri = 2;
			else {

				korelValues[26] = KorelFunction.korelFunction(tri, 2,
						KorelFunction.EQUAL_OP);
				korelValues[27] = KorelFunction.korelFunction(tri, 2,
						KorelFunction.NOT_EQUAL_OP);
				korelValues[28] = KorelFunction.korelFunction(i+k, j,
						KorelFunction.GREATER_THEN_OP);
				korelValues[29] = KorelFunction.korelFunction(i+k, j,
						KorelFunction.LESS_EQUAL_OP);
				if (
						(isReached[13]=(tri == 2) && (tri == 2))
						&&
						(isReached[14]=(i + k > j) && (i + k > j))
					)
					tri = 2;
				else {
					

					korelValues[30] = KorelFunction.korelFunction(tri, 3,
							KorelFunction.EQUAL_OP);
					korelValues[31] = KorelFunction.korelFunction(tri, 3,
							KorelFunction.NOT_EQUAL_OP);
					korelValues[32] = KorelFunction.korelFunction(j+k, i,
							KorelFunction.GREATER_THEN_OP);
					korelValues[33] = KorelFunction.korelFunction(j+k, i,
							KorelFunction.LESS_EQUAL_OP);
					if (
							(isReached[15]=(tri == 3) && (tri == 3))
							&& 
							(isReached[16]=(j + k > i) && (j + k > i))
						)
						tri = 2;
					else
						tri = 4;
				}
			}
		}
		return tri;
	}
//	
//	public static void main(String[] args){
//		Triangle t = new Triangle();
//		t.triangle(1, 1, 2);
//		Boolean[] reach = t.getIsReached();
//		Double[] korel = t.getKorelValues();
//		
//		for(int i = 0; i < reach.length; i++){
//			System.out.println(reach[i] +" - "+ korel[2*i] + " - "+ korel[2*i + 1]);
//		}
//		
//	}
}